/*
 * Created on Oct 27, 2004
 *
 */
package sync;

/**
 * An object of this class acts as a message source. It has a predefined
 * communication pattern: send data at a certain rate, the rate changes to show
 * the effect of a buffer between sender and receiver.
 * <p>
 * You could use this object as follows. Note that sender and receiver share a
 * common SyncPort object.
 * <p>
 * 
 * <code>
 *   SyncPort port = new SyncPort();<br>
 *   Sender s = new Sender(port);<br>
 *   Receiver r = new Receiver(port);<br>
 * </code>
 * <p>
 * Note that the sender and receiver threads are automatically started by the
 * constructors of the objects.
 * <p>
 * The sender always sends a message '9999' as the last one. This will cause the
 * receiver to terminate the program.
 * 
 * @author hvogt
 *         <p>
 *  
 */
public class Sender implements Runnable {

    SyncPort sp;

    int pattern[][] = { { 10, 250 }, { 5, 10 }, { 5, 250 }, { 2, 1000 }, { 5, 250 } };

    long waitingTime = 0;

    /**
     * The constructor for a sender object.
     * 
     * @param p
     *            The port over which the sender will send its messages.
     */
    public Sender(SyncPort p) {
        sp = p;
        new Thread(this).start();
    }

    /**
     * The thread loop.
     */
    public void run() {
        long t0 = System.currentTimeMillis();
        try {
            run_ex();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        long t1 = System.currentTimeMillis();
        System.out.println("Sender total running time: " + (t1 - t0) + " ms");
    }

    void doSend(int value) {
        long t0 = System.currentTimeMillis();
        sp.send(value);
        long t1 = System.currentTimeMillis();
        long blocked = t1 - t0;
        waitingTime += blocked;
        System.out.println("SEND(" + value + ") blocked for " + blocked + " ms");
    }

    void run_ex() throws Exception {
        int value = 1;
        for (int i = 0; i < pattern.length; i++) {
            int iterations = pattern[i][0];
            int delay = pattern[i][1];
            for (int k = 0; k < iterations; k++) {
                doSend(value++);
                Thread.sleep(delay);
            }
        }

        doSend(9999);
        System.out.println("Sender total blocked time: " + waitingTime + " ms");
    }

}