/*
 * Created on Oct 27, 2004
 *
 */
package sync;

/**
 * This class provides a synchronous communication mechanism. The sender blocks
 * until a receiver is available. Also, the receiver blocks until a sender is
 * available. You can exchange integers as messages.
 *  
 * @author hvogt<p>
 * 
 */
public class SyncPort {

    int buffer;

    boolean ready_send = false;
    boolean ready_receive = false;

    /**
     * Send a message. This method blocks the caller until a receiver is ready.
     * 
     * @param n The message to send (only integers in this version).
     */
    public synchronized void send(int n) {
        buffer = n;
        ready_send = true;
        if (ready_receive) {
            this.notifyAll();
        } else {
            try {
                this.wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        ready_send = false;
    }

    /**
     * Receive a message. This method blocks the caller until a sender is ready.
     * 
     * @return The message received (only integers in this version).
     */
    public synchronized int receive() {
        ready_receive = true;
        if (ready_send) {
            this.notifyAll();
        } else {
            try {
                this.wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        ready_receive = false;
        return buffer;
    }

}